local _favoriteButtonHeight = 30
local _mainWindowControl
local _minimizedWindowControl

local _isMinimizedLocked = false

local _fragment
local _minimizedFragment
local _isFragmentAddedToScenes = false
local _isMinimizedFragmentAddedToScenes = false

local _modes = {}
local _modeIndex = 0
local _favoritesModeIndex
local _emoteListModeIndex
local _personalityModeIndex

local _favoriteButtons = {}
local _selectedFavoriteIndex = 0

local _buttonTemplates = {
	"LE_FavoriteEmoteTextButtonTemplate",
	"LE_FavoriteEmoteDefaultButtonTemplate",
}

local function HasFavoriteMode()
	return _modeIndex == _favoritesModeIndex
end

local function IsMinimized()
	return _mainWindowControl:IsHidden()
end
LovelyEmotes.MainWindow.IsMinimized = IsMinimized

function LovelyEmotes.MainWindow.IsMinimizedLocked()
	return _isMinimizedLocked
end

function LovelyEmotes.MainWindow.SetMinimizedLocked(value)
	_isMinimizedLocked = value
end

local function AddFragmentToScenes(fragment)
	HUD_SCENE:AddFragment(fragment)
	HUD_UI_SCENE:AddFragment(fragment)
end

local function RemoveFragmentFromScenes(fragment)
	HUD_SCENE:RemoveFragment(fragment)
	HUD_UI_SCENE:RemoveFragment(fragment)
end

local function TryAddFragmentToScenes()
	if _isFragmentAddedToScenes or _isMinimizedFragmentAddedToScenes then return end
	_isFragmentAddedToScenes = true

	AddFragmentToScenes(_fragment)
end

local function TryAddMinimizedFragmentToScenes()
	if _isMinimizedFragmentAddedToScenes or _isFragmentAddedToScenes then return end
	_isMinimizedFragmentAddedToScenes = true

	AddFragmentToScenes(_minimizedFragment)
end
LovelyEmotes.MainWindow.TryAddMinimizedFragmentToScenes = TryAddMinimizedFragmentToScenes

local function TryRemoveFragmentFromScenes()
	if not _isFragmentAddedToScenes then return end
	_isFragmentAddedToScenes = false

	RemoveFragmentFromScenes(_fragment)
end

local function TryRemoveMinimizedFragmentFromScenes()
	if not _isMinimizedFragmentAddedToScenes then return end
	_isMinimizedFragmentAddedToScenes = false

	RemoveFragmentFromScenes(_minimizedFragment)
end
LovelyEmotes.MainWindow.TryRemoveMinimizedFragmentFromScenes = TryRemoveMinimizedFragmentFromScenes

local function SetWindowHeight(height)
	local guiRootHeight = GuiRoot:GetHeight()
	local savedAccountVariables = LovelyEmotes.SavedAccountVariables
	height = height + LE_Const_MainWindowBaseHeight

	if _mainWindowControl:GetTop() + height > guiRootHeight then
		_mainWindowControl:ClearAnchors()
		_mainWindowControl:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, savedAccountVariables.MainWindowPositionX, guiRootHeight - height)
	else
		_mainWindowControl:ClearAnchors()
		_mainWindowControl:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, savedAccountVariables.MainWindowPositionX, savedAccountVariables.MainWindowPositionY)
	end

	_mainWindowControl:SetHeight(height)
end
LovelyEmotes.MainWindow.SetWindowHeight = SetWindowHeight

local function Minimize()
	LovelyEmotes.PlayWindowCloseSound()

	_mainWindowControl:SetHidden(true)
	TryRemoveFragmentFromScenes()

	if LovelyEmotes.SavedAccountVariables.EnableMinimizedState == false then
		_minimizedWindowControl:SetHidden(true)
		TryRemoveMinimizedFragmentFromScenes()
		return
	end

	_minimizedWindowControl:SetHidden(false)
	TryAddMinimizedFragmentToScenes()
end

local function Expand()
	LovelyEmotes.PlayWindowOpenSound()

	_mainWindowControl:SetHidden(false)
	_minimizedWindowControl:SetHidden(true)

	TryRemoveMinimizedFragmentFromScenes()
	TryAddFragmentToScenes()
end

local function SetMinimized(value, setLocked)
	if IsMinimized() == value then return end

	if value then
		Minimize()
	else
		if LE_Invisible:IsHidden() or LovelyEmotes.RadialMenu.IsActive then return end
		Expand()
	end

	if (setLocked) then _isMinimizedLocked = value end
end
LovelyEmotes.MainWindow.SetMinimized = SetMinimized

function LovelyEmotes.MainWindow.ToggleMinimized(toggleCursor)
	if LE_Invisible:IsHidden() then return end

	SetMinimized(not IsMinimized(), true)

	if toggleCursor and SCENE_MANAGER:IsInUIMode() == IsMinimized() then
		SCENE_MANAGER:OnToggleHUDUIBinding()
	end
end

--[[
local function GetValidChannelID()
	if LovelyEmotes.SavedAccountVariables.EnablePartyChatSync and GetGroupSize() > 0 then return CHAT_CHANNEL_PARTY
	elseif LovelyEmotes.SavedAccountVariables.EnableSayChatSync then return CHAT_CHANNEL_SAY end
	--elseif LovelyEmotes.SavedAccountVariables.EnableZoneChatSync then return CHAT_CHANNEL_ZONE end

	return nil
end

local function SetValidChannel()
	local currentChannelID = CHAT_SYSTEM:GetCurrentChannelData().id

	if ((currentChannelID == CHAT_CHANNEL_SAY and LovelyEmotes.SavedAccountVariables.EnableSayChatSync) or
		(currentChannelID == CHAT_CHANNEL_PARTY and LovelyEmotes.SavedAccountVariables.EnablePartyChatSync) or
		((currentChannelID == CHAT_CHANNEL_WHISPER or currentChannelID == CHAT_CHANNEL_WHISPER_SENT) and LovelyEmotes.SavedAccountVariables.EnableWhisperChatSync) or
		(currentChannelID == CHAT_CHANNEL_ZONE and LovelyEmotes.SavedAccountVariables.EnableZoneChatSync)) then
		return
	end

	local newChannelID = GetValidChannelID()
	if newChannelID ~= nil then CHAT_SYSTEM:SetChannel(newChannelID) end
end
]]

local function DeselectFavoriteButton()
	if _selectedFavoriteIndex < 1 then return end

	local control = _favoriteButtons[_selectedFavoriteIndex]:GetNamedChild("EmoteButton")
	control:SetNormalFontColor(LovelyEmotes.GetEmoteTextColor(LovelyEmotes.GetSavedEmote(_selectedFavoriteIndex)))
	control:SetDisabledFontColor(ZO_DISABLED_TEXT:UnpackRGBA())
	_selectedFavoriteIndex = 0
end
LovelyEmotes.MainWindow.DeselectFavoriteButton = DeselectFavoriteButton

local function SelectFavoriteButton(index, control)
	if _selectedFavoriteIndex == index then return end
	if _selectedFavoriteIndex > 0 then DeselectFavoriteButton() end

	control:SetNormalFontColor(LOVELYEMOTES_SELECTED_FAVORITE_COLOR:UnpackRGBA())
	control:SetDisabledFontColor(LOVELYEMOTES_SELECTED_FAVORITE_COLOR:UnpackRGBA())
	_selectedFavoriteIndex = index
end

local function RefreshFavoriteButton(index)
	local targetControl = _favoriteButtons[index]
	local emoteControl = targetControl:GetNamedChild("EmoteButton")
	local targetEmote = LovelyEmotes.GetSavedEmote(index)

	targetControl:GetNamedChild("ConfigButton"):SetHandler("OnClicked", function()
		SelectFavoriteButton(index, emoteControl)
		LovelyEmotes.EmoteSelectionWindow.ShowWindow(index, targetEmote)
	end)

	if not targetEmote then
		emoteControl:SetEnabled(false)

		if (targetEmote == nil) then
			emoteControl:SetNormalFontColor(ZO_NORMAL_TEXT:UnpackRGBA())
			emoteControl:SetText(LE_Const_EmoteUnknownDisplayName)
			emoteControl:SetHandler("OnClicked", nil)
			return
		end

		--emoteControl:SetNormalFontColor(ZO_NORMAL_TEXT:UnpackRGBA())
		emoteControl:SetText("")
		emoteControl:SetHandler("OnClicked", nil)
		return
	end

	emoteControl:SetEnabled(true)

	if _selectedFavoriteIndex ~= index then
		emoteControl:SetNormalFontColor(LovelyEmotes.GetEmoteTextColor(targetEmote))
	end

	emoteControl:SetText(LovelyEmotes.GetEmoteDisplayName(targetEmote))
	emoteControl:SetHandler("OnClicked", function(control, button)
		if not LovelyEmotes.IsEmoteSynchronizationActive() or button == 1 or targetEmote.ID < 0 then
			targetEmote.Play()
			return
		end

		--SetValidChannel()
		CHAT_SYSTEM:StartTextEntry(zo_strformat("LovelyEmotes <<1>> <<2>>", targetEmote.ID, targetEmote.SlashName))
	end)
end
LovelyEmotes.MainWindow.RefreshFavoriteButton = RefreshFavoriteButton

local function RefreshAllFavoriteButtons()
	if not HasFavoriteMode() then return end

	for i = LovelyEmotes.GetFavoriteButtonCount(), 1, -1 do
		RefreshFavoriteButton(i)
	end
end
LovelyEmotes.MainWindow.RefreshAllFavoriteButtons = RefreshAllFavoriteButtons

local function CreateFavoriteButtons()
	local buttonTemplate = _buttonTemplates[LovelyEmotes.SavedAccountVariables.FavoritesButtonDesign]

	for i = 1, LE_Const_FavoriteEmotesIndexMax do
		local offsetY = _favoriteButtonHeight * (i - 1) + LE_Const_MainWindowBaseHeight
		local targetControl = CreateControlFromVirtual("LE_FavoriteEmoteButton", _mainWindowControl, buttonTemplate, i)
		targetControl:SetHeight(_favoriteButtonHeight)
		targetControl:ClearAnchors()
		targetControl:SetAnchor(TOPLEFT, nil, nil, 0, offsetY)
		targetControl:SetAnchor(TOPRIGHT, nil, nil, 0, offsetY)

		table.insert(_favoriteButtons, targetControl)
	end
end

local function SetFavoriteButtonsVisible()
	if not HasFavoriteMode() then return end

	local visibleButtonsCount = LovelyEmotes.GetFavoriteButtonCount()
	SetWindowHeight(_favoriteButtonHeight * visibleButtonsCount)

	for i = 1, LE_Const_FavoriteEmotesIndexMax do
		local isButtonHidden = i > visibleButtonsCount
		local favoriteButton = _favoriteButtons[i]

		if isButtonHidden then
			if favoriteButton:IsControlHidden() then return end
			favoriteButton:SetHidden(true)
		else
			-- Creates a random favorite-emote if required.
			if LovelyEmotes.GetSavedID(i) == nil then
				LovelyEmotes.SaveRandomEmote(i)
			end

			if favoriteButton:IsControlHidden() then
				favoriteButton:SetHidden(false)
			end
		end
	end
end
LovelyEmotes.MainWindow.SetFavoriteButtonsVisible = SetFavoriteButtonsVisible

local function HideAllFavoriteButtons()
	for i = 1, LE_Const_FavoriteEmotesIndexMax do
		local favoriteButton = _favoriteButtons[i]

		if not favoriteButton:IsControlHidden() then
			favoriteButton:SetHidden(true)
		end
	end
end

local function SetTabButtonAlpha(index, alpha)
	local control = GetControl("LE_MainWindowControlTabButton", index)
	if not control then return end

	local r, g, b = ZO_NORMAL_TEXT:UnpackRGB()
	control:SetNormalFontColor(r, g, b, alpha)
end

local function UpdateTabButtonsAlpha()
	local tabIndex = LovelyEmotes.GetActiveFavoriteEmotesTabIndex()

	for i = LovelyEmotes.GetNumberOfVisibleFavoriteEmotesTabs(), 1, -1 do
		if i == tabIndex then
			SetTabButtonAlpha(i, 1)
		else
			SetTabButtonAlpha(i, 0.33)
		end
	end
end

local function SetTabButtonsHidden(isHidden)
	local numberOfFavoriteEmotesTabs = LovelyEmotes.GetNumberOfVisibleFavoriteEmotesTabs()

	if numberOfFavoriteEmotesTabs < 2 then
		isHidden = true
	end

	for i = LE_Const_FavoriteEmotesTabIndexMax, 1, -1 do
		local control = GetControl("LE_MainWindowControlTabButton", i)

		if control ~= nil then
			if isHidden == true then
				control:SetHidden(isHidden)
			else
				control:SetHidden(i > numberOfFavoriteEmotesTabs)
			end
		end
	end
end

function LovelyEmotes.MainWindow.UpdateTabButtonsHidden()
	if HasFavoriteMode() == true then
		SetTabButtonsHidden(false)
	end

	UpdateTabButtonsAlpha()
end

local function ShowFavoriteEmotesTab(tabIndex, playSound)
	LovelyEmotes.EmoteSelectionWindow.HideWindow()

	SetTabButtonAlpha(LovelyEmotes.GetActiveFavoriteEmotesTabIndex(), 0.33)
	SetTabButtonAlpha(tabIndex, 1)

	LovelyEmotes.SetActiveFavoriteEmotesTabIndex(tabIndex)
	SetFavoriteButtonsVisible()
	RefreshAllFavoriteButtons()

	if playSound then LovelyEmotes.PlayTabSound() end
end
LovelyEmotes.MainWindow.ShowFavoriteEmotesTab = ShowFavoriteEmotesTab

function LovelyEmotes.MainWindow.TryShowTab(tabIndex, playSound)
	if tabIndex == LovelyEmotes.GetActiveFavoriteEmotesTabIndex() then
		return false
	end

	ShowFavoriteEmotesTab(tabIndex, playSound)
	return true
end

function LovelyEmotes.MainWindow.ShowNextTab()
	local tabIndex = LovelyEmotes.GetActiveFavoriteEmotesTabIndex() + 1

	if tabIndex > LovelyEmotes.GetNumberOfVisibleFavoriteEmotesTabs() then
		tabIndex = 1
	end

	ShowFavoriteEmotesTab(tabIndex, true)
end

function LovelyEmotes.MainWindow.ShowPreviousTab()
	local tabIndex = LovelyEmotes.GetActiveFavoriteEmotesTabIndex() - 1

	if tabIndex < 1 then
		tabIndex = LovelyEmotes.GetNumberOfVisibleFavoriteEmotesTabs()
	end

	ShowFavoriteEmotesTab(tabIndex, true)
end

local function OpenFavoritesMode()
	SetTabButtonsHidden(false)
	SetFavoriteButtonsVisible()
	RefreshAllFavoriteButtons()
end

local function CloseFavoritesMode()
	LovelyEmotes.EmoteSelectionWindow.HideWindow(true)
	SetTabButtonsHidden(true)
	HideAllFavoriteButtons()
end

local function OpenEmoteListMode()
	SetWindowHeight(450)

	LovelyEmotes.EmoteList:SetParent(_mainWindowControl, 0, function(button, data)
		if not LovelyEmotes.IsEmoteSynchronizationActive() or button == 1 or data.ID < 0 then
			data.Play()
			return
		end

		CHAT_SYSTEM:StartTextEntry(zo_strformat("LovelyEmotes <<1>> <<2>>", data.ID, data.SlashName))
	end)

	LovelyEmotes.EmoteList:ResetList()
end

local function CloseEmoteListMode()
	LovelyEmotes.EmoteSelectionWindow.SetupEmoteList()
end

local function OpenMode(index)
	if _modeIndex == index then return end

	if _modeIndex > 0 then
		_modes[_modeIndex].CloseFunction()
	end

	_modeIndex = index
	_modes[index].OpenFunction()
end

local function AddMode(openFunction, closeFunction)
	table.insert(_modes, {
		OpenFunction = openFunction,
		CloseFunction = closeFunction,
	} )

	return #_modes
end

local function ToggleMode(modeIndex)
	if _modeIndex ~= modeIndex then
		OpenMode(modeIndex)
		return
	end

	OpenMode(_favoritesModeIndex)
end

function LovelyEmotes.MainWindow.ToggleEmoteListMode()
	ToggleMode(_emoteListModeIndex)
end

function LovelyEmotes.MainWindow.TogglePersonalityListMode()
	ToggleMode(_personalityModeIndex)
end

function LovelyEmotes.MainWindow.UpdateAlpha()
	_mainWindowControl:SetAlpha(LovelyEmotes.SavedAccountVariables.Alpha / 100)
end

function LovelyEmotes.MainWindow.UpdateAlphaMinimized()
	_minimizedWindowControl:SetAlpha(LovelyEmotes.SavedAccountVariables.MinimizedAlpha / 100)
end

local function SavePosition()
	local savedAccountVariables = LovelyEmotes.SavedAccountVariables
	savedAccountVariables.MainWindowPositionX = _mainWindowControl:GetLeft()
	savedAccountVariables.MainWindowPositionY = _mainWindowControl:GetTop()
end

local function SaveMinimizedPosition()
	local savedAccountVariables = LovelyEmotes.SavedAccountVariables
	savedAccountVariables.MainWindowMinimizedPositionX = _minimizedWindowControl:GetLeft()
	savedAccountVariables.MainWindowMinimizedPositionY = _minimizedWindowControl:GetTop()
end

local function SetMinimizedWindowScale(newScale)
	_minimizedWindowControl:SetScale(newScale)
end
LovelyEmotes.MainWindow.SetMinimizedWindowScale = SetMinimizedWindowScale

local function CreateTabs()
	for i = 1, LE_Const_FavoriteEmotesTabIndexMax do
		local buttonControl = CreateControlFromVirtual("LE_MainWindowControlTabButton", _mainWindowControl, "LE_TabButtonTemplate", i)
		buttonControl:ClearAnchors()
		buttonControl:SetAnchor(TOPLEFT, LE_MainWindowControl, TOPLEFT, 44 + buttonControl:GetWidth() * (i - 1), 0)

		buttonControl:SetText(i)
		buttonControl:SetHandler("OnClicked", function()
			if LovelyEmotes.MainWindow.TryShowTab(i, true) == false then
				PlaySound("Click")
			end
		end)
	end
end

function LovelyEmotes.MainWindow.Initialize()
	_mainWindowControl = LE_MainWindowControl
	_minimizedWindowControl = LE_MinimizedWindowControl
	local savedAccountVariables = LovelyEmotes.SavedAccountVariables

	_mainWindowControl:ClearAnchors()
	_mainWindowControl:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, savedAccountVariables.MainWindowPositionX, savedAccountVariables.MainWindowPositionY)
	_mainWindowControl:SetHandler("OnMoveStop", SavePosition)

	_minimizedWindowControl:ClearAnchors()
	_minimizedWindowControl:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, savedAccountVariables.MainWindowMinimizedPositionX, savedAccountVariables.MainWindowMinimizedPositionY)
	_minimizedWindowControl:SetHandler("OnMoveStop", SaveMinimizedPosition)

	_favoritesModeIndex = AddMode(OpenFavoritesMode, CloseFavoritesMode)
	_emoteListModeIndex = AddMode(OpenEmoteListMode, CloseEmoteListMode)
	_personalityModeIndex = AddMode(LovelyEmotes.PersonalityList.GetFunctions())
	_modeIndex = _favoritesModeIndex

	CreateFavoriteButtons()
	CreateTabs()

	SetFavoriteButtonsVisible()
	RefreshAllFavoriteButtons()
	LovelyEmotes.MainWindow.UpdateTabButtonsHidden()

	LovelyEmotes.MainWindow.UpdateAlpha()
	LovelyEmotes.MainWindow.UpdateAlphaMinimized()

	SetMinimizedWindowScale(savedAccountVariables.MinimizedScale)

	_fragment = ZO_SimpleSceneFragment:New(_mainWindowControl)
	_minimizedFragment = ZO_SimpleSceneFragment:New(_minimizedWindowControl)

	if savedAccountVariables.StartMinimized or savedAccountVariables.MinimizeAutomatically then
		SetMinimized(true, true)
	else
		Expand()
	end
end
